#ifndef DEBUG_
#define DEBUG_

//======================================================================
//
// Debugging facilities
//
// Copyright (C 1999 CurveSoft, Inc. All rights reserved.
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
// USA.
//  
// $Revision: 1.6 $
//
//======================================================================

// Need ostream
#ifndef _FSTREAM_H
#include <fstream.h>
#endif

//
// Debug objects
// -- Each object must be initialized with a file name containing debug
//    flags and debug output file names.
// -- Each line of the initialization file must have no more than
//    (maxline-1) chars and has the following format (all lines
//    beginning with # are treated as comments):
//      flagname 0|1 file
//    where "flagname" must be one of the values of "flagNames",
//    the 0 or 1 turn the flag off and on, "file" is the name of the
//    file to redirect debug output controlled by this flag (if the file
//    is missing and the flag is 1, output goes to stdout).
//
class Debug {
public:
    static const int maxLine = 150 ;

    // Keep in alphabetic order and in sync with flagNames in Debug.C
    enum dbgEnum {
        eConic, eStrokeFont,
        eLast   // Not a debug flag; *MUST* be the last one.
    } ;

    // ctor
    Debug( char const *const f ) ;

    // dtor
    ~Debug() ;

    // other methods
    bool
    Flag( dbgEnum i ) { return mFlags[ i ] ; } ;

    ostream &
    File( dbgEnum i ) {
        if ( eLast == mFileIndex[ i ] ) {
            return cout ;
        } else {
            return mOfile[ mFileIndex[ i ] ] ;
        }
    } ;

    void Dump() ;

 private:
    char*         mInitFile ;
    bool          mFlags[ eLast ] ;
    static char*  mFlagNames[ eLast ] ;
    char*         mFileNames[ eLast ] ;
    // Next slot in mFileNames, mFileIndex, and mOfile
    dbgEnum       mNextSlot ;

    // If mFlags[ i ] is on, the output stream is:
    //     "cout" if mFileIndex[ i ] is eLast;
    //     mOfile[ mFileIndex[ i ] ] otherwise
    dbgEnum  mFileIndex[ eLast ] ;
    ofstream mOfile[ eLast ] ;

    // open file and initialize mFlags
    void Init( char const *const f ) ;
    void BasicInit() ;
} ;

#ifdef DEBUG
extern Debug dbg ;
#endif

#endif  // DEBUG_
