#=======================================================================
#
# Procedures for output of font as PS Type 3 font
#
# Copyright (C 1999 CurveSoft, Inc. All rights reserved.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
# USA.
#  
# $Revision: 1.13 $
#
#=======================================================================

#-------------------------------------------------------------  WrapupSf
#
# Output entire font as Type 3 font
#
# -- Open output file and write out font header to it
# -- Write out all character definitions
# -- Write out code to define the Type 3 font
#
proc WrapupSf {} {
    global aName; upvar #0 $aName G

    # open output file and write out font header
    set G(fileId) [open $G(psFile) w]
    puts $G(fileId) [read [open $G(psHead) r]]

    # write out all character definitions
    set clist [lsort [info procs $G(prefix)@\[0-9a-f\]*$G(suffix)]]
    puts $G(fileId) "[expr 1 + [llength $clist]] dict\
        begin\n/.notdef \{ 0 0    0 0 0 0 _p6 \} bd"
    foreach i $clist { $i }

    # wrapup
    puts $G(fileId) "currentdict end /CharStrings exch put\
        dup /FontName get exch definefont pop"
}

#-------------------------------------------------------------------  S_
#
# Start char definition
#
# Output CharString beginning (CharStrings dictionary should be on the
# dictionary stack)
#
proc S_ {g wx wy llx lly urx ury nSeg nStr} {
    global aName; upvar #0 $aName G
    set i [lindex [info level [expr [info level] - 1]] 0]

    if ![regexp (^$G(prefix)@)(\[0-9a-f\]+)($G(suffix)$) \
            $i z x gg y] {
        puts stderr "ERROR(S_): Bad char proc ($i)"; exit 1
    }
    if {$g != $gg} {
        puts stderr "ERROR(S_): Bad code ($gg != $g)"; exit 1
    }
    puts $G(fileId) "/_${g} \{ $wx $wy    $llx $lly $urx $ury _p6"
}

#-------------------------------------------------------------------  E_
#
# End char definition
#
proc E_ {} {
    global aName; upvar #0 $aName G
    puts $G(fileId) "  stroke \} bd"
}

#-------------------------------------------------------------------  M_
#
# moveto
#
proc M_ {x y} {
    global aName; upvar #0 $aName G
    puts $G(fileId) "  $x $y _xf _m"
}

#-------------------------------------------------------------------  L_
#
# lineto
#
proc L_ {x y} {
    global aName; upvar #0 $aName G
    puts $G(fileId) "  $x $y _xf _l"
}

#-------------------------------------------------------------------  C_
#
# conicto
#
proc C_ {x y x1 y1 b} {
    global aName; upvar #0 $aName G
    puts $G(fileId) "  $x $y _xf $x1 $y1 _xf $b _c"
}
